@@ -1,7 +1,7 @@
 /* ------------------------------------------------------------ */
 /*
 HTTrack Website Copier, Offline Browser for Windows and Unix
-Copyright (C) 1998-2013 Xavier Roche and other contributors
+Copyright (C) 1998-2014 Xavier Roche and other contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -54,12 +54,13 @@ Please visit our Website: http://www.httrack.com
 
 #endif
 
-typedef struct hostent FAR t_hostent;
-
 #else
 #define HTS_USESCOPEID
 #define INVALID_SOCKET -1
-typedef struct hostent t_hostent;
+#endif
+
+#ifdef __cplusplus
+extern "C" {
 #endif
 
 #if HTS_USEOPENSSL
@@ -157,4 +158,8 @@ typedef enum HTTrackStatus {
   STATUS_FTP_READY = 1001
 } HTTrackStatus;
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
