@@ -1,7 +1,7 @@
 /* ------------------------------------------------------------ */
 /*
 HTTrack Website Copier, Offline Browser for Windows and Unix
-Copyright (C) 1998-2013 Xavier Roche and other contributors
+Copyright (C) 1998-2014 Xavier Roche and other contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -33,8 +33,19 @@ Please visit our Website: http://www.httrack.com
 #include "htscharset.h"
 #include "htsbase.h"
 #include "punycode.h"
+#include "htssafe.h"
 
-#include <assert.h>
+#ifdef _WIN32
+#include <stddef.h>
+typedef unsigned __int32 uint32_t;
+typedef unsigned __int64 uint64_t;
+#elif (defined(SOLARIS) || defined(sun) || defined(HAVE_INTTYPES_H) \
+  || defined(BSD) || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(__FreeBSD_kernel__))
+#include <inttypes.h>
+#else
+#include <stdint.h>
+#endif
+#include <stdarg.h>
 
 int hts_isStringAscii(const char *s, size_t size) {
   size_t i;
@@ -415,7 +426,7 @@ static char* hts_codepageToUTF8(const char *codepage, const char *s) {
       if (dest != NULL) {
         const size_t len = hts_writeUTF8(uc, &dest[k], MAX_UTF);
         k += len;
-        assert(k < capa);
+        assertf(k < capa);
       }
     }
     dest[k] = '\0';
@@ -442,7 +453,7 @@ static char *hts_convertStringCharset(const char *s, size_t size,
     const iconv_t cp = iconv_open(to, from);
 
     if (cp != (iconv_t) - 1) {
-      char *inbuf = (char *) s;
+      char *inbuf = (char*) (uintptr_t) s; /* ugly iconv api, sheesh */
       size_t inbytesleft = size;
       size_t outbufCapa = 0;
       char *outbuf = NULL;
@@ -724,43 +735,21 @@ static unsigned int nlz8(unsigned char x) {
   26	U+200000	U+3FFFFFF	5	111110xx
   31	U+4000000	U+7FFFFFFF	6	1111110x
 */
-#define ADD_SEQ(UC, BITS, EMITTER) do {                                 \
-    /* number of data bits in first octet */                            \
-    const int bits = BITS % 6;                                          \
-    /* shift for first octet */                                         \
-    const int shift0 = BITS - bits;                                     \
-    /* first octet */                                                   \
-    const unsigned char lead =                                          \
-      /* leading bits */                                                \
-      ( 0xff ^ ( ( 1 << ( bits + 1 ) ) - 1 ) )                          \
-      /* encoded bits */                                                \
-      | ( ( (UC) >> shift0 ) & ( ( 1 << ( bits + 1 ) ) - 1 ) )          \
-      ;                                                                 \
-    /* further bytes are encoding 6 bits */                             \
-    const unsigned char second =                                        \
-      0x80 | ( ( (UC) >> ( shift0 - 6 ) ) & 0x3f );                     \
-    EMITTER(lead);                                                      \
-    EMITTER(second);                                                    \
-    if (BITS > 6*2) {                                                   \
-      const unsigned char next =                                        \
-        0x80 | ( ( (UC) >> ( shift0 - 6*2 ) ) & 0x3f );                 \
-      EMITTER(next);                                                    \
-      if (BITS > 6*3) {                                                 \
-        const unsigned char next =                                      \
-          0x80 | ( ( (UC) >> ( shift0 - 6*3 ) ) & 0x3f );               \
-        EMITTER(next);                                                  \
-        if (BITS > 6*4) {                                               \
-          const unsigned char next =                                    \
-            0x80 | ( ( (UC) >> ( shift0 - 6*4 ) ) & 0x3f );             \
-          EMITTER(next);                                                \
-          if (BITS > 6*5) {                                             \
-            const unsigned char next =                                  \
-              0x80 | ( ( (UC) >> ( shift0 - 6*5 ) ) & 0x3f );           \
-            EMITTER(next);                                              \
-          }                                                             \
-        }                                                               \
-      }                                                                 \
-    }                                                                   \
+#define ADD_FIRST_SEQ(UC, LEN, EMITTER) do {                            \
+  /* first octet */                                                     \
+  const unsigned char lead =                                            \
+    /* leading bits: LEN "1" bits */                                    \
+    ~ ( ( 1 << (unsigned) ( 8 - LEN ) ) - 1 )                           \
+    /* encoded bits */                                                  \
+    | ( (UC) >> (unsigned) ( ( LEN - 1 ) * 6 ) );                       \
+  EMITTER(lead);                                                        \
+  } while(0)
+
+#define ADD_NEXT_SEQ(UC, SHIFT, EMITTER) do {                           \
+  /* further bytes are encoding 6 bits */                               \
+  const unsigned char next =                                            \
+    0x80 | ( ( (UC) >> SHIFT ) & 0x3f );                                \
+  EMITTER(next);                                                        \
   } while(0)
 
 /* UC is a constant. EMITTER is a macro function taking an unsigned int. */
@@ -768,15 +757,30 @@ static unsigned int nlz8(unsigned char x) {
     if ((UC) < 0x80) {                          \
       EMITTER(((unsigned char) (UC)));          \
     } else if ((UC) < 0x0800) {                 \
-      ADD_SEQ(UC, 11, EMITTER);                 \
+      ADD_FIRST_SEQ(UC, 2, EMITTER);            \
+      ADD_NEXT_SEQ(UC, 0, EMITTER);             \
     } else if ((UC) < 0x10000) {                \
-      ADD_SEQ(UC, 16, EMITTER);                 \
+      ADD_FIRST_SEQ(UC, 3, EMITTER);            \
+      ADD_NEXT_SEQ(UC, 6, EMITTER);             \
+      ADD_NEXT_SEQ(UC, 0, EMITTER);             \
     } else if ((UC) < 0x200000) {               \
-      ADD_SEQ(UC, 21, EMITTER);                 \
+      ADD_FIRST_SEQ(UC, 4, EMITTER);            \
+      ADD_NEXT_SEQ(UC, 12, EMITTER);            \
+      ADD_NEXT_SEQ(UC, 6, EMITTER);             \
+      ADD_NEXT_SEQ(UC, 0, EMITTER);             \
     } else if ((UC) < 0x4000000) {              \
-      ADD_SEQ(UC, 26, EMITTER);                 \
+      ADD_FIRST_SEQ(UC, 5, EMITTER);            \
+      ADD_NEXT_SEQ(UC, 18, EMITTER);            \
+      ADD_NEXT_SEQ(UC, 12, EMITTER);            \
+      ADD_NEXT_SEQ(UC, 6, EMITTER);             \
+      ADD_NEXT_SEQ(UC, 0, EMITTER);             \
     } else {                                    \
-      ADD_SEQ(UC, 31, EMITTER);                 \
+      ADD_FIRST_SEQ(UC, 6, EMITTER);            \
+      ADD_NEXT_SEQ(UC, 24, EMITTER);            \
+      ADD_NEXT_SEQ(UC, 18, EMITTER);            \
+      ADD_NEXT_SEQ(UC, 12, EMITTER);            \
+      ADD_NEXT_SEQ(UC, 6, EMITTER);             \
+      ADD_NEXT_SEQ(UC, 0, EMITTER);             \
     }                                           \
   } while(0)
 
@@ -967,31 +971,36 @@ char *hts_convertStringUTF8ToIDNA(const char *s, size_t size) {
             if (HTS_IS_LEADING_UTF8(c)) {
               /* commit sequence ? */
               if (utfSeq != (size_t) -1) {
-                /* unicode character */
-                punycode_uint uc = 0;
 
                 /* Reader: can read bytes up to j */
 #define RD ( utfSeq < j ? segData[utfSeq++] : -1 )
 
                 /* Writer: upon error, return FFFD (replacement character) */
-#define WR(C) uc = C != -1 ? (punycode_uint) C : (punycode_uint) 0xfffd
+#define WR(C) do { \
+  if ((C) != -1) { \
+    /* copy character */ \
+    assertf(segOutputSize < segSize); \
+    segInt[segOutputSize++] = (C); \
+  } \
+  /* In case of error, abort. */ \
+  else { \
+    FREE_BUFFER(); \
+    return NULL; \
+  } \
+} while(0)
 
-                /* Read Unicode character. */
+                /* Read/Write Unicode character. */
                 READ_UNICODE(RD, WR);
 #undef RD
 #undef WR
 
-                /* copy character */
-                assert(segOutputSize < segSize);
-                segInt[segOutputSize++] = uc;
-
                 /* not anymore in sequence */
                 utfSeq = (size_t) -1;
               }
 
               /* ascii ? */
               if (c < 0x80) {
-                assert(segOutputSize < segSize);
+                assertf(segOutputSize < segSize);
                 segInt[segOutputSize] = c;
                 if (c != 0) {
                   segOutputSize++;
