@@ -1,7 +1,7 @@
 /* ------------------------------------------------------------ */
 /*
 HTTrack Website Copier, Offline Browser for Windows and Unix
-Copyright (C) 1998-2013 Xavier Roche and other contributors
+Copyright (C) 1998-2014 Xavier Roche and other contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -36,8 +36,8 @@ Please visit our Website: http://www.httrack.com
 #define HTTRACK_GLOBAL_DEFH
 
 // Version (also check external version information)
-#define HTTRACK_VERSION      "3.48-3"
-#define HTTRACK_VERSIONID    "3.48.3"
+#define HTTRACK_VERSION      "3.48-19"
+#define HTTRACK_VERSIONID    "3.48.19"
 #define HTTRACK_AFF_VERSION  "3.x"
 #define HTTRACK_LIB_VERSION  "2.0"
 
@@ -57,6 +57,19 @@ Please visit our Website: http://www.httrack.com
 #endif
 #endif
 
+/* GCC extension */
+#ifndef HTS_UNUSED
+#ifdef __GNUC__
+#define HTS_UNUSED __attribute__ ((unused))
+#define HTS_STATIC static __attribute__ ((unused))
+#define HTS_PRINTF_FUN(fmt, arg) __attribute__ ((format (printf, fmt, arg)))
+#else
+#define HTS_UNUSED
+#define HTS_STATIC static
+#define HTS_PRINTF_FUN(fmt, arg)
+#endif
+#endif
+
 // config.h
 #ifdef _WIN32
 
@@ -119,11 +132,6 @@ Please visit our Website: http://www.httrack.com
 
 #endif
 
-// don't spare memory usage by default
-#ifndef HTS_SPARE_MEMORY
-#define HTS_SPARE_MEMORY 0
-#endif
-
 #ifndef BIGSTK
 #define BIGSTK
 #endif
@@ -193,20 +201,13 @@ Please visit our Website: http://www.httrack.com
 
 #endif
 
-#if HTS_SPARE_MEMORY==0
 /* Taille max d'une URL */
 #define HTS_URLMAXSIZE 1024
 /* Taille max ligne de commande (>=HTS_URLMAXSIZE*2) */
 #define HTS_CDLMAXSIZE 1024
-#else
-/* Taille max d'une URL */
-#define HTS_URLMAXSIZE 256
-/* Taille max ligne de commande (>=HTS_URLMAXSIZE*2) */
-#define HTS_CDLMAXSIZE 1024
-#endif
 
-/* Copyright (C) 1998-2013 Xavier Roche and other contributors */
-#define HTTRACK_AFF_AUTHORS "[XR&CO'2013]"
+/* Copyright (C) 1998-2014 Xavier Roche and other contributors */
+#define HTTRACK_AFF_AUTHORS "[XR&CO'2014]"
 #define HTS_DEFAULT_FOOTER "<!-- Mirrored from %s%s by HTTrack Website Copier/"HTTRACK_AFF_VERSION" "HTTRACK_AFF_AUTHORS", %s -->"
 #define HTTRACK_WEB "http://www.httrack.com"
 #define HTS_UPDATE_WEBSITE "http://www.httrack.com/update.php3?Product=HTTrack&Version="HTTRACK_VERSIONID"&VersionStr="HTTRACK_VERSION"&Platform=%d&Language=%s"
@@ -244,8 +245,14 @@ Please visit our Website: http://www.httrack.com
 #define HTSEXT_API __declspec(dllimport)
 #endif
 #else
+/* See <http://gcc.gnu.org/wiki/Visibility> */
+#if ( ( defined(__GNUC__) && ( __GNUC__ >= 4 ) ) \
+      || ( defined(HAVE_VISIBILITY) && HAVE_VISIBILITY ) )
+#define HTSEXT_API __attribute__ ((visibility ("default")))
+#else
 #define HTSEXT_API
 #endif
+#endif
 
 #ifndef HTS_LONGLONG
 #ifdef HTS_NO_64_BIT
@@ -316,11 +323,6 @@ typedef unsigned __int32 T_SOC;
 typedef int T_SOC;
 #endif
 
-/* Default alignement */
-#ifndef HTS_ALIGN
-#define HTS_ALIGN (sizeof(void*))
-#endif
-
 /* IPV4, IPV6 and various unified structures */
 #define HTS_MAXADDRLEN 64
 
@@ -365,11 +367,7 @@ typedef int T_SOC;
 #define LOCAL_SOCKET_ID -2
 
 // taille de chaque buffer (10 sockets 650 ko)
-#if HTS_SPARE_MEMORY==0
 #define TAILLE_BUFFER 65536
-#else
-#define TAILLE_BUFFER 8192
-#endif
 
 #ifdef HTS_DO_NOT_USE_PTHREAD
 #error needs threads support
