@@ -1,7 +1,7 @@
 /* ------------------------------------------------------------ */
 /*
 HTTrack Website Copier, Offline Browser for Windows and Unix
-Copyright (C) 1998-2013 Xavier Roche and other contributors
+Copyright (C) 1998-2014 Xavier Roche and other contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -52,7 +52,7 @@ Please visit our Website: http://www.httrack.com
 /* END specific definitions */
 
 #define waitkey if (more) { char s[4]; printf("\nMORE.. q to quit\n"); linput(stdin,s,4); if (strcmp(s,"q")==0) quit=1; else printf("Page %d\n\n",++m); }
-void infomsg(char *msg) {
+void infomsg(const char *msg) {
   int l = 0;
   int m = 0;
   int more = 0;
@@ -116,16 +116,51 @@ void infomsg(char *msg) {
     }
   }
 }
+
+typedef struct help_wizard_buffers {
+  char urls[HTS_URLMAXSIZE * 2];
+  char mainpath[256];
+  char projname[256];
+  char stropt[2048];        // options
+  char stropt2[2048];       // options longues
+  char strwild[2048];       // wildcards
+  char cmd[4096];
+  char str[256];
+  char *argv[256];
+} help_wizard_buffers;
+
 void help_wizard(httrackp * opt) {
-  char *urls = (char *) malloct(HTS_URLMAXSIZE * 2);
-  char *mainpath = (char *) malloct(256);
-  char *projname = (char *) malloct(256);
-  char *stropt = (char *) malloct(2048);        // options
-  char *stropt2 = (char *) malloct(2048);       // options longues
-  char *strwild = (char *) malloct(2048);       // wildcards
-  char *cmd = (char *) malloct(4096);
-  char *str = (char *) malloct(256);
-  char **argv = (char **) malloct(256 * sizeof(char *));
+  help_wizard_buffers *buffers = malloct(sizeof(help_wizard_buffers));
+
+#undef urls
+#undef mainpath
+#undef projname
+#undef stropt
+#undef stropt2
+#undef strwild
+#undef cmd
+#undef str
+#undef argv
+
+#define urls (buffers->urls)
+#define mainpath (buffers->mainpath)
+#define projname (buffers->projname)
+#define stropt (buffers->stropt)
+#define stropt2 (buffers->stropt2)
+#define strwild (buffers->strwild)
+#define cmd (buffers->cmd)
+#define str (buffers->str)
+#define argv (buffers->argv)
+
+  //char *urls = (char *) malloct(HTS_URLMAXSIZE * 2);
+  //char *mainpath = (char *) malloct(256);
+  //char *projname = (char *) malloct(256);
+  //char *stropt = (char *) malloct(2048);        // options
+  //char *stropt2 = (char *) malloct(2048);       // options longues
+  //char *strwild = (char *) malloct(2048);       // wildcards
+  //char *cmd = (char *) malloct(4096);
+  //char *str = (char *) malloct(256);
+  //char **argv = (char **) malloct(256 * sizeof(char *));
 
   //
   char *a;
@@ -147,7 +182,7 @@ void help_wizard(httrackp * opt) {
   printf("\n");
   printf("Welcome to HTTrack Website Copier (Offline Browser) " HTTRACK_VERSION
          "%s\n", hts_get_version_info(opt));
-  printf("Copyright (C) 1998-2013 Xavier Roche and other contributors\n");
+  printf("Copyright (C) 1998-2014 Xavier Roche and other contributors\n");
 #ifdef _WIN32
   printf("Note: You are running the commandline version,\n");
   printf("run 'WinHTTrack.exe' to get the GUI version.\n");
@@ -237,7 +272,7 @@ void help_wizard(httrackp * opt) {
     if (strnotempty(str)) {
       while((a = strchr(str, ' ')))
         *a = ':';               // port
-      if (!strchr(jump_identification(str), ':')) {
+      if (!strchr(jump_identification_const(str), ':')) {
         char str2[256];
 
         printf("\nProxy port (return=8080) :");
@@ -297,7 +332,7 @@ void help_wizard(httrackp * opt) {
       printf("\n");
 
       // couper en morceaux
-      argv[0] = "winhttrack";
+      argv[0] = strdup("winhttrack");
       argv[1] = cmd;
       argc++;
       while(cmd[i]) {
@@ -318,24 +353,26 @@ void help_wizard(httrackp * opt) {
   }
 
   /* Free buffers */
-  freet(urls);
-  freet(mainpath);
-  freet(projname);
-  freet(stropt);
-  freet(stropt2);
-  freet(strwild);
-  freet(cmd);
-  freet(str);
+  free(buffers);
+#undef urls
+#undef mainpath
+#undef projname
+#undef stropt
+#undef stropt2
+#undef strwild
+#undef cmd
+#undef str
+#undef argv
 }
-int help_query(char *list, int def) {
+int help_query(const char *list, int def) {
   char s[256];
-  char *a;
+  const char *a;
   int opt;
   int n = 1;
 
   a = list;
   while(strnotempty(a)) {
-    char *b = strchr(a, '|');
+    const char *b = strchr(a, '|');
 
     if (b) {
       char str[256];
@@ -401,7 +438,7 @@ void help_catchurl(const char *dest_path) {
       {
         char BIGSTK finalurl[HTS_URLMAXSIZE * 2];
 
-        escape_check_url(dest);
+        inplace_escape_check_url(dest, sizeof(dest));
         sprintf(finalurl, "%s" POSTTOK "file:%s", url, dest);
         printf("\nThe URL is: \"%s\"\n", finalurl);
         printf("You can capture it through: httrack \"%s\"\n", finalurl);
@@ -418,7 +455,7 @@ void help_catchurl(const char *dest_path) {
 }
 
 // Créer un index.html vide
-void make_empty_index(char *str) {
+void make_empty_index(const char *str) {
 #if 0
   if (!fexist(fconcat(str, "index.html"))) {
     FILE *fp = fopen(fconcat(str, "index.html"), "wb");
@@ -436,7 +473,7 @@ void make_empty_index(char *str) {
 
 // mini-aide  (h: help)
 //           y
-void help(char *app, int more) {
+void help(const char *app, int more) {
   char info[2048];
 
   infomsg("");
@@ -444,7 +481,7 @@ void help(char *app, int more) {
     infomsg("1");
   if (more != 2) {
     sprintf(info,
-            "HTTrack version " HTTRACK_VERSION "%s (compiled " __DATE__ ")",
+            "HTTrack version " HTTRACK_VERSION "%s",
             hts_is_available());
     infomsg(info);
 #ifdef HTTRACK_AFF_WARNING
@@ -568,7 +605,7 @@ void help(char *app, int more) {
   infomsg
     (" %F  footer string in Html code (-%F \"Mirrored [from host %s [file %s [at %s]]]\"");
   infomsg(" %l  preffered language (-%l \"fr, en, jp, *\"");
-  infomsg(" %a  accepted formats (-%l \"text/html,image/png,image/jpeg,image/gif;q=0.9,*/*;q=0.1\"");
+  infomsg(" %a  accepted formats (-%a \"text/html,image/png;q=0.9,*/*;q=0.1\"");
   infomsg(" %X  additional HTTP header line (-%X \"X-Magic: 42\"");
   infomsg("");
   infomsg("Log, index, cache");
@@ -755,10 +792,10 @@ void help(char *app, int more) {
   infomsg("example: httrack --continue");
   infomsg("continues a mirror in the current folder");
   infomsg("");
-  sprintf(info, "HTTrack version " HTTRACK_VERSION "%s (compiled " __DATE__ ")",
+  sprintf(info, "HTTrack version " HTTRACK_VERSION "%s",
           hts_is_available());
   infomsg(info);
-  infomsg("Copyright (C) 1998-2013 Xavier Roche and other contributors");
+  infomsg("Copyright (C) 1998-2014 Xavier Roche and other contributors");
 #ifdef HTS_PLATFORM_NAME
   infomsg("[compiled: " HTS_PLATFORM_NAME "]");
 #endif
