@@ -1,7 +1,7 @@
 /* ------------------------------------------------------------ */
 /*
 HTTrack Website Copier, Offline Browser for Windows and Unix
-Copyright (C) 1998-2013 Xavier Roche and other contributors
+Copyright (C) 1998-2014 Xavier Roche and other contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -35,6 +35,8 @@ Please visit our Website: http://www.httrack.com
 #ifndef HTS_DEFH
 #define HTS_DEFH
 
+#include "httrack-library.h"
+
 /* Forward definitions */
 #ifndef HTS_DEF_FWSTRUCT_htsrequest
 #define HTS_DEF_FWSTRUCT_htsrequest
@@ -48,6 +50,14 @@ typedef struct htsblk htsblk;
 #define HTS_DEF_FWSTRUCT_t_dnscache
 typedef struct t_dnscache t_dnscache;
 #endif
+#ifndef HTS_DEF_FWSTRUCT_lien_adrfil
+#define HTS_DEF_FWSTRUCT_lien_adrfil
+typedef struct lien_adrfil lien_adrfil;
+#endif
+#ifndef HTS_DEF_FWSTRUCT_lien_adrfilsave
+#define HTS_DEF_FWSTRUCT_lien_adrfilsave
+typedef struct lien_adrfilsave lien_adrfilsave;
+#endif
 
 /* définitions globales */
 #include "htsglobal.h"
@@ -92,87 +102,8 @@ MSVC2003INLINEBUG HTS_STATIC char *getHtsOptBuff_(httrackp * opt) {
 }
 
 #undef MSVC2003INLINEBUG
-#define OPT_GET_BUFF(OPT) ( getHtsOptBuff_(OPT) )
-
-// structure pour paramètres supplémentaires lors de la requête
-#ifndef HTS_DEF_FWSTRUCT_htsrequest_proxy
-#define HTS_DEF_FWSTRUCT_htsrequest_proxy
-typedef struct htsrequest_proxy htsrequest_proxy;
-#endif
-struct htsrequest_proxy {
-  int active;
-  const char* name;
-  int port;
-  const char* bindhost;           // bind this host
-};
-
-#ifndef HTS_DEF_FWSTRUCT_htsrequest
-#define HTS_DEF_FWSTRUCT_htsrequest
-typedef struct htsrequest htsrequest;
-#endif
-struct htsrequest {
-  short int user_agent_send;    // user agent (ex: httrack/1.0 [sun])
-  short int http11;             // l'en tête peut (doit) être signé HTTP/1.1 et non HTTP/1.0
-  short int nokeepalive;        // pas de keep-alive
-  short int range_used;         // Range utilisé
-  short int nocompression;      // Pas de compression
-  short int flush_garbage;      // recycled
-  const char* user_agent;
-  const char* referer;
-  const char* from;
-  const char* lang_iso;
-  const char* accept;
-  const char* headers;
-  htsrequest_proxy proxy;       // proxy
-};
-
-// structure pour retour d'une connexion/prise d'en tête
-#ifndef HTS_DEF_FWSTRUCT_htsblk
-#define HTS_DEF_FWSTRUCT_htsblk
-typedef struct htsblk htsblk;
-#endif
-struct htsblk {
-  int statuscode;               // status-code, -1=erreur, 200=OK,201=..etc (cf RFC1945)
-  short int notmodified;        // page ou fichier NON modifié (transféré)
-  short int is_write;           // sortie sur disque (out) ou en mémoire (adr)
-  short int is_chunk;           // mode chunk
-  short int compressed;         // compressé?
-  short int empty;              // vide?
-  short int keep_alive;         // Keep-Alive?
-  short int keep_alive_trailers;        // ..with trailers extension
-  int keep_alive_t;             // KA timeout
-  int keep_alive_max;           // KA number of requests
-  char *adr;                    // adresse du bloc de mémoire, NULL=vide
-  char *headers;                // adresse des en têtes si présents
-  FILE *out;                    // écriture directe sur disque (si is_write=1)
-  LLint size;                   // taille fichier
-  char msg[80];                 // message éventuel si échec ("\0"=non précisé)
-  char contenttype[64];         // content-type ("text/html" par exemple)
-  char charset[64];             // charset ("iso-8859-1" par exemple)
-  char contentencoding[64];     // content-encoding ("gzip" par exemple)
-  char *location;               // on copie dedans éventuellement la véritable 'location'
-  LLint totalsize;              // taille totale à télécharger (-1=inconnue)
-  short int is_file;            // ce n'est pas une socket mais un descripteur de fichier si 1
-  T_SOC soc;                    // ID socket
-  SOCaddr address;              // IP address
-  int address_size;             // IP address structure length
-  FILE *fp;                     // fichier pour file://
-#if HTS_USEOPENSSL
-  short int ssl;                // is this connection a SSL one? (https)
-  // BIO* ssl_soc;          // SSL structure
-  SSL *ssl_con;                 // connection structure
-#endif
-  char lastmodified[64];        // Last-Modified
-  char etag[256];               // Etag
-  char cdispo[256];             // Content-Disposition coupé
-  LLint crange;                 // Content-Range
-  LLint crange_start;           // Content-Range
-  LLint crange_end;             // Content-Range
-  int debugid;                  // debug connection
-  /* */
-  htsrequest req;               // paramètres pour la requête
-  /*char digest[32+2];   // digest md5 généré par le moteur ("" si non généré) */
-};
+#define OPT_GET_BUFF(OPT)      ( getHtsOptBuff_(OPT) )
+#define OPT_GET_BUFF_SIZE(OPT) ( sizeof(opt->state.concat.buff[0]) )
 
 /* ANCIENNE STURCTURE pour cache 1.0 */
 #ifndef HTS_DEF_FWSTRUCT_OLD_t_proxy
@@ -217,12 +148,10 @@ struct OLD_htsblk {
 typedef struct t_dnscache t_dnscache;
 #endif
 struct t_dnscache {
-  struct t_dnscache *n;
-  int host_length;              // 4 normalement - ==0  alors en cours de résolution
-  char host_addr[HTS_MAXADDRLEN];       // 4 octets (v4), ou 16 octets (v6)
-  // ou >16 si sockaddr
-  //                 ==-1 alors erreur (host n'éxiste pas)
-  char iadr[1024];
+  struct t_dnscache *next;
+  const char *iadr;
+  size_t host_length;                   // length ; (4 or 16) ; 0 for error
+  char host_addr[HTS_MAXADDRLEN];
 };
 
 /* Library internal definictions */
@@ -231,6 +160,9 @@ struct t_dnscache {
 // initialize an htsblk structure
 void hts_init_htsblk(htsblk * r);
 
+// attach specific project log to hachtable logger
+void hts_set_hash_handler(coucal hashtable, httrackp *opt);
+
 // version
 HTSEXT_API const char* hts_version(void);
 
@@ -240,31 +172,15 @@ int hts_read(htsblk * r, char *buff, int size);
 //int HTS_TOTAL_RECV_CHECK(int var);
 LLint check_downloadable_bytes(int rate);
 
-#ifndef HTTRACK_DEFLIB
-HTSEXT_API int hts_init(void);
-HTSEXT_API int hts_uninit(void);
 HTSEXT_API int hts_uninit_module(void);
-HTSEXT_API int hts_resetvar(void);      /* dummy */
-HTSEXT_API void hts_debug(int level);
-HTSEXT_API httrackp *hts_create_opt(void);
-HTSEXT_API void hts_free_opt(httrackp * opt);
-HTSEXT_API const hts_stat_struct* hts_get_stats(httrackp * opt);
-HTSEXT_API void set_wrappers(httrackp * opt);   /* LEGACY */
-HTSEXT_API int plug_wrapper(httrackp * opt, const char *moduleName,
-                            const char *argv);
-
-HTSEXT_API char *hts_strdup(const char *string);
-HTSEXT_API void *hts_malloc(size_t size);
-HTSEXT_API void *hts_realloc(void *data, size_t size);
-HTSEXT_API void hts_free(void *data);
-#endif
 
 // fonctions principales
-T_SOC http_fopen(httrackp * opt, char *adr, char *fil, htsblk * retour);
+T_SOC http_fopen(httrackp * opt, const char *adr, const char *fil, htsblk * retour);
 T_SOC http_xfopen(httrackp * opt, int mode, int treat, int waitconnect,
-                  char *xsend, char *adr, char *fil, htsblk * retour);
-int http_sendhead(httrackp * opt, t_cookie * cookie, int mode, char *xsend,
-                  char *adr, char *fil, char *referer_adr, char *referer_fil,
+                  const char *xsend, const char *adr, const char *fil, htsblk * retour);
+int http_sendhead(httrackp * opt, t_cookie * cookie, int mode, const char *xsend,
+                  const char *adr, const char *fil,
+                  const char *referer_adr, const char *referer_fil,
                   htsblk * retour);
 
 //int newhttp(char* iadr,char* err=NULL);
@@ -274,47 +190,39 @@ HTS_INLINE void deletehttp(htsblk * r);
 HTS_INLINE int deleteaddr(htsblk * r);
 HTS_INLINE void deletesoc(T_SOC soc);
 HTS_INLINE void deletesoc_r(htsblk * r);
-htsblk http_location(httrackp * opt, char *adr, char *fil, char *loc);
-htsblk http_test(httrackp * opt, char *adr, char *fil, char *loc);
+htsblk http_test(httrackp * opt, const char *adr, const char *fil, char *loc);
 int check_readinput(htsblk * r);
 int check_readinput_t(T_SOC soc, int timeout);
-void treathead(t_cookie * cookie, char *adr, char *fil, htsblk * retour,
+void treathead(t_cookie * cookie, const char *adr, const char *fil, htsblk * retour,
                char *rcvd);
-void treatfirstline(htsblk * retour, char *rcvd);
-
-#ifndef HTTRACK_DEFLIB
-HTSEXT_API void infostatuscode(char *msg, int statuscode);
-#endif
+void treatfirstline(htsblk * retour, const char *rcvd);
 
 // sous-fonctions
 LLint http_xfread1(htsblk * r, int bufl);
-HTS_INLINE t_hostent *hts_gethostbyname2(httrackp * opt, const char *iadr,
-                                        void *v_buffer, const char **error);
-HTS_INLINE t_hostent *hts_gethostbyname(httrackp * opt, const char *iadr,
-                                        void *v_buffer);
-#ifndef HTTRACK_DEFLIB
-HTSEXT_API t_hostent *vxgethostbyname2(char *hostname, void *v_buffer, const char **error);
-HTSEXT_API t_hostent *vxgethostbyname(char *hostname, void *v_buffer);
-HTSEXT_API int check_hostname_dns(char *hostname);
-#endif
+HTS_INLINE SOCaddr* hts_dns_resolve2(httrackp * opt, const char *iadr,
+                                     SOCaddr *const addr, 
+                                     const char **error);
+HTS_INLINE SOCaddr* hts_dns_resolve(httrackp * opt, const char *iadr,
+                                    SOCaddr *const addr);
+HTSEXT_API SOCaddr* hts_dns_resolve_nocache2(const char *const hostname, 
+                                              SOCaddr *const addr,
+                                              const char **error);
+HTSEXT_API SOCaddr* hts_dns_resolve_nocache(const char *const hostname, 
+                                             SOCaddr *const addr);
+HTSEXT_API int check_hostname_dns(const char *const hostname);
+
 int ftp_available(void);
 
 #if HTS_DNSCACHE
-void hts_cache_free(t_dnscache * cache);
-t_dnscache *_hts_cache(httrackp * opt);
+void hts_cache_free(t_dnscache *const cache);
+t_dnscache *hts_cache(httrackp * opt);
 #endif
 
 // outils divers
 HTS_INLINE TStamp time_local(void);
 
-#ifndef HTTRACK_DEFLIB
-HTSEXT_API HTS_INLINE TStamp mtime_local(void);
-#endif
 void sec2str(char *s, TStamp t);
 
-#ifndef HTTRACK_DEFLIB
-HTSEXT_API void qsec2str(char *st, TStamp t);
-#endif
 void time_gmt_rfc822(char *s);
 void time_local_rfc822(char *s);
 struct tm *convert_time_rfc822(struct tm *buffer, const char *s);
@@ -324,14 +232,8 @@ int get_filetime_rfc822(const char *file, char *date);
 HTS_INLINE void time_rfc822(char *s, struct tm *A);
 HTS_INLINE void time_rfc822_local(char *s, struct tm *A);
 
-#ifndef HTTRACK_DEFLIB
-HTSEXT_API char *int2char(strc_int2bytes2 * strc, int n);
-HTSEXT_API char *int2bytes(strc_int2bytes2 * strc, LLint n);
-HTSEXT_API char *int2bytessec(strc_int2bytes2 * strc, long int n);
-HTSEXT_API char **int2bytes2(strc_int2bytes2 * strc, LLint n);
-#endif
 HTS_INLINE int sendc(htsblk * r, const char *s);
-int finput(int fd, char *s, int max);
+int finput(T_SOC fd, char *s, int max);
 int binput(char *buff, char *s, int max);
 int linput(FILE * fp, char *s, int max);
 int linputsoc(T_SOC soc, char *s, int max);
@@ -339,136 +241,81 @@ int linputsoc_t(T_SOC soc, char *s, int max, int timeout);
 int linput_trim(FILE * fp, char *s, int max);
 int linput_cpp(FILE * fp, char *s, int max);
 void rawlinput(FILE * fp, char *s, int max);
-char *strstrcase(char *s, char *o);
-int ident_url_absolute(const char *url, char *adr, char *fil);
+const char *strstrcase(const char *s, const char *o);
+int ident_url_absolute(const char *url, lien_adrfil *adrfil);
 void fil_simplifie(char *f);
-int is_unicode_utf8(const char *buffer, size_t size);
+int is_unicode_utf8(const char *buffer, const size_t size);
 void map_characters(unsigned char *buffer, unsigned int size,
                     unsigned int *map);
 int ishtml(httrackp * opt, const char *urlfil);
 int ishtml_ext(const char *a);
 int ishttperror(int err);
-void guess_httptype(httrackp * opt, char *s, const char *fil);
 
-#ifndef HTTRACK_DEFLIB
-HTSEXT_API void get_httptype(httrackp * opt, char *s, const char *fil,
-                             int flag);
-#endif
 int get_userhttptype(httrackp * opt, char *s, const char *fil);
 void give_mimext(char *s, const char *st);
 
-#ifndef HTTRACK_DEFLIB
-HTSEXT_API int is_knowntype(httrackp * opt, const char *fil);
-HTSEXT_API int is_userknowntype(httrackp * opt, const char *fil);
-HTSEXT_API int is_dyntype(const char *fil);
-HTSEXT_API char *get_ext(char *catbuff, const char *fil);
-#endif
-int may_unknown(httrackp * opt, const char *st);
 int may_bogus_multiple(httrackp * opt, const char *mime, const char *filename);
 int may_unknown2(httrackp * opt, const char *mime, const char *filename);
 
-#ifndef HTTRACK_DEFLIB
-HTSEXT_API char *jump_identification(const char *);
-HTSEXT_API char *jump_normalized(const char *);
-HTSEXT_API char *jump_toport(const char *);
-HTSEXT_API char *fil_normalized(const char *source, char *dest);
-HTSEXT_API char *adr_normalized(const char *source, char *dest);
-#endif
-char *strrchr_limit(const char *s, char c, const char *limit);
-char *strstr_limit(const char *s, const char *sub, const char *limit);
-HTS_INLINE char *jump_protocol(const char *source);
+const char *strrchr_limit(const char *s, char c, const char *limit);
+char *jump_protocol(char *source);
+const char *jump_protocol_const(const char *source);
 void code64(unsigned char *a, int size_a, unsigned char *b, int crlf);
 
-#ifndef HTTRACK_DEFLIB
-HTSEXT_API void unescape_amp(char *s);
-HTSEXT_API void escape_spc_url(char *s);
-HTSEXT_API void escape_in_url(char *s);
-HTSEXT_API void escape_uri(char *s);
-HTSEXT_API void escape_uri_utf(char *s);
-HTSEXT_API void escape_check_url(char *s);
-HTSEXT_API char *escape_check_url_addr(char *catbuff, const char *s);
-HTSEXT_API void x_escape_http(char *s, int mode);
-HTSEXT_API void x_escape_html(char *s);
-HTSEXT_API void escape_remove_control(char *s);
-HTSEXT_API void escape_for_html_print(char *s, char *d);
-HTSEXT_API void escape_for_html_print_full(char *s, char *d);
-#endif
-#ifndef HTTRACK_DEFLIB
-HTSEXT_API char *unescape_http(char *catbuff, const char *s);
-HTSEXT_API char *unescape_http_unharm(char *catbuff, const char *s,
-                                      int no_high);
-HTSEXT_API char *antislash_unescaped(char *catbuff, const char *s);
-HTSEXT_API char *concat(char *catbuff, const char *a, const char *b);
-HTSEXT_API char *fconcat(char *catbuff, const char *a, const char *b);
-HTSEXT_API char *fconv(char *catbuff, const char *a);
-#endif
 #define copychar(catbuff,a) concat(catbuff,(a),NULL)
-char *fslash(char *catbuff, const char *a);
 
-#ifndef HTTRACK_DEFLIB
-HTSEXT_API int hts_log(httrackp * opt, const char *prefix, const char *msg);
-HTSEXT_API void hts_log_print(httrackp * opt, int type, const char *format,
-                              ...) HTS_PRINTF_FUN(3, 4);
-#endif
+char *convtolower(char *catbuff, const char *a);
+void hts_lowcase(char *s);
+void hts_replace(char *s, char from, char to);
+int multipleStringMatch(const char *s, const char *match);
 
-     char *convtolower(char *catbuff, const char *a);
-     void hts_lowcase(char *s);
-     void hts_replace(char *s, char from, char to);
-     int multipleStringMatch(const char *s, const char *match);
-
-     void fprintfio(FILE * fp, char *buff, char *prefix);
+void fprintfio(FILE * fp, const char *buff, const char *prefix);
 
 #ifdef _WIN32
 #else
-     int sig_ignore_flag(int setflag);  // flag ignore
+int sig_ignore_flag(int setflag);  // flag ignore
 #endif
 
-     void cut_path(char *fullpath, char *path, char *pname);
-     int fexist(const char *s);
-     int fexist_utf8(const char *s);
+void cut_path(char *fullpath, char *path, char *pname);
+int fexist(const char *s);
+int fexist_utf8(const char *s);
 
 /*LLint fsize(const char* s);    */
-     off_t fpsize(FILE * fp);
-     off_t fsize(const char *s);
-     off_t fsize_utf8(const char *s);
-
-/* root dir */
-#ifndef HTTRACK_DEFLIB
-     HTSEXT_API char *hts_rootdir(char *file);
-#endif
+off_t fpsize(FILE * fp);
+off_t fsize(const char *s);
+off_t fsize_utf8(const char *s);
 
 // Threads
-     typedef void *(*beginthread_type) (void *);
+typedef void *(*beginthread_type) (void *);
 
 /*unsigned long _beginthread( beginthread_type start_address, unsigned stack_size, void *arglist );*/
 
 /* variables globales */
-     extern HTSEXT_API hts_stat_struct HTS_STAT;
-     extern int _DEBUG_HEAD;
-     extern FILE *ioinfo;
+extern HTSEXT_API hts_stat_struct HTS_STAT;
+extern int _DEBUG_HEAD;
+extern FILE *ioinfo;
 
 /* constantes */
-     extern const char *hts_mime_keep[];
-     extern const char *hts_mime[][2];
-     extern const char *hts_main_mime[];
-     extern const char *hts_detect[];
-     extern const char *hts_detectbeg[];
-     extern const char *hts_nodetect[];
-     extern const char *hts_detectURL[];
-     extern const char *hts_detectandleave[];
-     extern const char *hts_detect_js[];
+extern const char *hts_mime_keep[];
+extern const char *hts_mime[][2];
+extern const char *hts_main_mime[];
+extern const char *hts_detect[];
+extern const char *hts_detectbeg[];
+extern const char *hts_nodetect[];
+extern const char *hts_detectURL[];
+extern const char *hts_detectandleave[];
+extern const char *hts_detect_js[];
 
 // htsmodule.c definitions
-     extern void *openFunctionLib(const char *file_);
-     extern void *getFunctionPtr(void *handle, const char *fncname);
-     extern void closeFunctionLib(void *handle);
+extern void *openFunctionLib(const char *file_);
+extern void *getFunctionPtr(void *handle, const char *fncname);
+extern void closeFunctionLib(void *handle);
 
-     extern void clearCallbacks(htscallbacks * chain);
-     extern size_t hts_get_callback_offs(const char *name);
-     int hts_set_callback(t_hts_htmlcheck_callbacks * callbacks,
-                          const char *name, void *function);
-     void *hts_get_callback(t_hts_htmlcheck_callbacks * callbacks,
-                            const char *name);
+extern void clearCallbacks(htscallbacks * chain);
+int hts_set_callback(t_hts_htmlcheck_callbacks * callbacks,
+                     const char *name, void *function);
+void *hts_get_callback(t_hts_htmlcheck_callbacks * callbacks,
+                       const char *name);
 
 #define CBSTRUCT(OPT) ((t_hts_htmlcheck_callbacks*) ((OPT)->callbacks_fun))
 #define GET_USERCALLBACK(OPT, NAME) ( CBSTRUCT(OPT)-> NAME .fun )
@@ -581,7 +428,7 @@ HTSEXT_API void hts_log_print(httrackp * opt, int type, const char *format,
 
 // compare le début de f avec s et retourne la position de la fin
 // 'A=a' (case insensitive)
-     HTS_STATIC int strfield(const char *f, const char *s) {
+HTS_STATIC int strfield(const char *f, const char *s) {
   int r = 0;
 
   while(streql(*f, *s) && ((*f) != 0) && ((*s) != 0)) {
@@ -593,8 +440,9 @@ HTSEXT_API void hts_log_print(httrackp * opt, int type, const char *format,
     return r;
   else
     return 0;
-     }
-HTS_STATIC int strcmpnocase(char *a, char *b) {
+}
+
+HTS_STATIC int strcmpnocase(const char *a, const char *b) {
   while(*a) {
     int cmp = hichar(*a) - hichar(*b);
 
@@ -689,6 +537,16 @@ HTS_STATIC int compare_mime(httrackp * opt, const char *mime, const char *file,
 
 #endif
 
+// returns (size_t) -1 upon error
+static HTS_UNUSED size_t off_t_to_size_t(off_t o) {
+  const size_t so = (size_t) o;
+  if ((off_t) so == o) {
+    return so;
+  } else {
+    return (size_t) -1;
+  }
+}
+
 /* dirent() compatibility */
 #ifdef _WIN32
 #define HTS_DIRENT_SIZE 256
