@@ -1,7 +1,7 @@
 /* ------------------------------------------------------------ */
 /*
 HTTrack Website Copier, Offline Browser for Windows and Unix
-Copyright (C) 1998-2013 Xavier Roche and other contributors
+Copyright (C) 1998-2014 Xavier Roche and other contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -67,21 +67,21 @@ Please visit our Website: http://www.httrack.com
 
 /* Bypass internal definition protection */
 #define HTS_INTERNAL_BYTECODE
-#include "htsinthash.h"
+#include "coucal.h"
 #undef HTS_INTERNAL_BYTECODE
 
 int NewLangStrSz = 1024;
-inthash NewLangStr = NULL;
+coucal NewLangStr = NULL;
 int NewLangStrKeysSz = 1024;
-inthash NewLangStrKeys = NULL;
+coucal NewLangStrKeys = NULL;
 int NewLangListSz = 1024;
-inthash NewLangList = NULL;
+coucal NewLangList = NULL;
 
 /* Language files */
 
 #include "htsserver.h"
 
-char *gethomedir(void);
+const char *gethomedir(void);
 int commandRunning = 0;
 int commandEndRequested = 0;
 int commandEnd = 0;
@@ -97,13 +97,13 @@ extern void webhttrack_main(char *cmd);
 extern void webhttrack_lock(void);
 extern void webhttrack_release(void);
 
-static int is_image(char *file) {
+static int is_image(const char *file) {
   return ((strstr(file, ".gif") != NULL));
 }
-static int is_text(char *file) {
+static int is_text(const char *file) {
   return ((strstr(file, ".txt") != NULL));
 }
-static int is_html(char *file) {
+static int is_html(const char *file) {
   return ((strstr(file, ".htm") != NULL));
 }
 
@@ -119,16 +119,16 @@ HTS_UNUSED static int linputsoc_t(T_SOC soc, char *s, int max, int timeout);
 HTS_UNUSED static int linput(FILE * fp, char *s, int max);
 
 /* Language files */
-HTS_UNUSED static int htslang_load(char *limit_to, char *apppath);
-HTS_UNUSED static void conv_printf(char *from, char *to);
+HTS_UNUSED static int htslang_load(char *limit_to, const char *apppath);
+HTS_UNUSED static void conv_printf(const char *from, char *to);
 HTS_UNUSED static void LANG_DELETE(void);
-HTS_UNUSED static void LANG_INIT(char *path);
-HTS_UNUSED static int LANG_T(char *path, int l);
+HTS_UNUSED static void LANG_INIT(const char *path);
+HTS_UNUSED static int LANG_T(const char *path, int l);
 HTS_UNUSED static int QLANG_T(int l);
-HTS_UNUSED static char *LANGSEL(char *name);
-HTS_UNUSED static char *LANGINTKEY(char *name);
-HTS_UNUSED static int LANG_SEARCH(char *path, char *iso);
-HTS_UNUSED static int LANG_LIST(char *path, char *buffer);
+HTS_UNUSED static const char *LANGSEL(const char *name);
+HTS_UNUSED static const char *LANGINTKEY(const char *name);
+HTS_UNUSED static int LANG_SEARCH(const char *path, const char *iso);
+HTS_UNUSED static int LANG_LIST(const char *path, char *buffer, size_t size);
 
 // URL Link catcher
 
@@ -162,16 +162,15 @@ T_SOC smallserver_init_std(int *port_prox, char *adr_prox, int defaultPort) {
 // 1- Init the URL catcher
 
 // get hostname. return 1 upon success.
-static int gethost(const char *hostname, SOCaddr * server, size_t server_size) {
+static int gethost(const char *hostname, SOCaddr * server) {
   if (hostname != NULL && *hostname != '\0') {
 #if HTS_INET6==0
     /* ipV4 resolver */
-    t_hostent *hp = gethostbyname(hostname);
+    struct hostent *hp = gethostbyname(hostname);
 
     if (hp != NULL) {
       if (hp->h_length) {
-        SOCaddr_copyaddr(*server, server_size, hp->h_addr_list[0],
-          hp->h_length);
+        SOCaddr_copyaddr2(*server, hp->h_addr_list[0], hp->h_length);
         return 1;
       }
     }
@@ -187,7 +186,7 @@ static int gethost(const char *hostname, SOCaddr * server, size_t server_size) {
     if (getaddrinfo(hostname, NULL, &hints, &res) == 0) {
       if (res) {
         if ((res->ai_addr) && (res->ai_addrlen)) {
-          SOCaddr_copyaddr(*server, server_size, res->ai_addr, res->ai_addrlen);
+          SOCaddr_copyaddr2(*server, res->ai_addr, res->ai_addrlen);
           freeaddrinfo(res);
           return 1;
         }
@@ -204,7 +203,7 @@ static int gethost(const char *hostname, SOCaddr * server, size_t server_size) {
 static int my_getlocalhost(char *h_loc, size_t size) {
   SOCaddr addr;
   strcpy(h_loc, "localhost");
-  if (gethost(h_loc, &addr, sizeof(addr)) == 1) {
+  if (gethost(h_loc, &addr) == 1) {
     return 0;
   }
   // come on ...
@@ -220,7 +219,7 @@ static int my_gethostname(char *h_loc, size_t size) {
   h_loc[0] = '\0';
   if (gethostname(h_loc, (int) size) == 0) {    // host name
     SOCaddr addr;
-    if (gethost(h_loc, &addr, sizeof(addr)) == 1) {
+    if (gethost(h_loc, &addr) == 1) {
       return 0;
     } else {
       return my_getlocalhost(h_loc, size);
@@ -247,54 +246,16 @@ T_SOC smallserver_init(int *port, char *adr) {
 
   if (my_gethostname(h_loc, 256) == 0) {   // host name
     SOCaddr server;
-    int server_size = sizeof(server);
-
-    /*t_hostent* hp_loc;
-       t_fullhostent buffer; */
-
-    // effacer structure
-    memset(&server, 0, sizeof(server));
-
-                                                                /*if ( (hp_loc=vxgethostbyname(h_loc, &buffer)) ) */
-    {
-      // copie adresse
-      // NO (bind all)
-      // SOCaddr_copyaddr(server, server_size, hp_loc->h_addr_list[0], hp_loc->h_length);
-
-      SOCaddr_initany(server, server_size);
-      if ((soc =
-           (T_SOC) socket(SOCaddr_sinfamily(server), SOCK_STREAM,
-                          0)) != INVALID_SOCKET) {
-        SOCaddr_initport(server, *port);
-        if (bind(soc, (struct sockaddr *) &server, server_size) == 0) {
-          /*SOClen len;
-             SOCaddr server2;
-             len=sizeof(server2); */
-          // effacer structure
-          /*memset(&server2, 0, sizeof(server2));
-             if (getsockname(soc,(struct sockaddr*) &server2,&len) == 0) {
-             *port=ntohs(SOCaddr_sinport(server));  // récupérer port */
-          if (listen(soc, 10) >= 0) {   // au pif le 10
-            // SOCaddr_inetntoa(adr, 128, server2, len);
-            strcpy(adr, h_loc);
-          } else {
-#ifdef _WIN32
-            closesocket(soc);
-#else
-            close(soc);
-#endif
-            soc = INVALID_SOCKET;
-          }
-
-          /*} else {
-             #ifdef _WIN32
-             closesocket(soc);
-             #else
-             close(soc);
-             #endif
-             soc=INVALID_SOCKET;
-             } */
 
+    SOCaddr_initany(server);
+    if ((soc =
+         (T_SOC) socket(SOCaddr_sinfamily(server), SOCK_STREAM,
+                        0)) != INVALID_SOCKET) {
+      SOCaddr_initport(server, *port);
+      if (bind(soc, &SOCaddr_sockaddr(server), SOCaddr_size(server)) == 0) {
+        if (listen(soc, 10) >= 0) {
+          // SOCaddr_inetntoa(adr, 128, server2);
+          strcpy(adr, h_loc);
         } else {
 #ifdef _WIN32
           closesocket(soc);
@@ -303,6 +264,13 @@ T_SOC smallserver_init(int *port, char *adr) {
 #endif
           soc = INVALID_SOCKET;
         }
+      } else {
+#ifdef _WIN32
+        closesocket(soc);
+#else
+        close(soc);
+#endif
+        soc = INVALID_SOCKET;
       }
     }
   }
@@ -319,16 +287,16 @@ T_SOC smallserver_init(int *port, char *adr) {
 // data: 32Kb
 
 typedef struct {
-  char *name;
+  const char *name;
   int value;
 } initIntElt;
 typedef struct {
-  char *name;
-  char *value;
+  const char *name;
+  const char *value;
 } initStrElt;
 
 #define SET_ERROR(err) do { \
-  inthash_write(NewLangList, "error", (intptr_t)strdup(err)); \
+  coucal_write(NewLangList, "error", (intptr_t)strdup(err)); \
   error_redirect = "/server/error.html"; \
 } while(0)
 
@@ -358,10 +326,10 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
   {
     char pth[1024];
 
-    char *initOn[] = { "parseall", "Cache", "ka",
+    const char *initOn[] = { "parseall", "Cache", "ka",
       "cookies", "parsejava", "testall", "updhack", "urlhack", "index", NULL
     };
-    initIntElt initInt[] = {
+    const initIntElt initInt[] = {
       {"filter", 4},
       {"travel", 2},
       {"travel2", 1},
@@ -381,8 +349,8 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
     initStrElt initStr[] = {
       {"user", "Mozilla/4.5 (compatible; HTTrack 3.0x; Windows 98)"},
       {"footer",
-       "<!-- Mirrored from %s%s by HTTrack Website Copier/3.x [XR&CO'2013], %s -->"},
-      {"url2", "+*.png +*.gif +*.jpg +*.css +*.js -ad.doubleclick.net/*"},
+       "<!-- Mirrored from %s%s by HTTrack Website Copier/3.x [XR&CO'2014], %s -->"},
+      {"url2", "+*.png +*.gif +*.jpg +*.jpeg +*.css +*.js -ad.doubleclick.net/*"},
       {NULL, NULL}
     };
     int i = 0;
@@ -391,18 +359,18 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
       char tmp[32];
 
       sprintf(tmp, "%d", initInt[i].value);
-      inthash_write(NewLangList, initInt[i].name, (intptr_t) strdup(tmp));
+      coucal_write(NewLangList, initInt[i].name, (intptr_t) strdup(tmp));
     }
     for(i = 0; initOn[i]; i++) {
-      inthash_write(NewLangList, initOn[i], (intptr_t) strdup("1"));    /* "on" */
+      coucal_write(NewLangList, initOn[i], (intptr_t) strdup("1"));    /* "on" */
     }
     for(i = 0; initStr[i].name; i++) {
-      inthash_write(NewLangList, initStr[i].name,
+      coucal_write(NewLangList, initStr[i].name,
                     (intptr_t) strdup(initStr[i].value));
     }
     strcpybuff(pth, gethomedir());
     strcatbuff(pth, "/websites");
-    inthash_write(NewLangList, "path", (intptr_t) strdup(pth));
+    coucal_write(NewLangList, "path", (intptr_t) strdup(pth));
   }
 
   /* Lock */
@@ -415,7 +383,7 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
     char line2[1024];
     T_SOC soc_c;
     LLint length = 0;
-    char *error_redirect = NULL;
+    const char *error_redirect = NULL;
 
     line[0] = '\0';
     buffer[0] = '\0';
@@ -500,15 +468,15 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
           char tmp[32];
 
           sprintf(tmp, "%d", commandReturn);
-          inthash_write(NewLangList, "commandReturn", (intptr_t) strdup(tmp));
-          inthash_write(NewLangList, "commandReturnMsg",
+          coucal_write(NewLangList, "commandReturn", (intptr_t) strdup(tmp));
+          coucal_write(NewLangList, "commandReturnMsg",
                         (intptr_t) commandReturnMsg);
-          inthash_write(NewLangList, "commandReturnCmdl",
+          coucal_write(NewLangList, "commandReturnCmdl",
                         (intptr_t) commandReturnCmdl);
         } else {
-          inthash_write(NewLangList, "commandReturn", (intptr_t) NULL);
-          inthash_write(NewLangList, "commandReturnMsg", (intptr_t) NULL);
-          inthash_write(NewLangList, "commandReturnCmdl", (intptr_t) NULL);
+          coucal_write(NewLangList, "commandReturn", (intptr_t) NULL);
+          coucal_write(NewLangList, "commandReturnMsg", (intptr_t) NULL);
+          coucal_write(NewLangList, "commandReturnCmdl", (intptr_t) NULL);
         }
       }
 
@@ -516,8 +484,8 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
       {
         intptr_t adr = 0;
 
-        if (inthash_readptr(NewLangList, "_sid", &adr)) {
-          if (inthash_write
+        if (coucal_readptr(NewLangList, "_sid", &adr)) {
+          if (coucal_write
               (NewLangList, "sid", (intptr_t) strdup((char *) adr))) {
           }
         }
@@ -530,7 +498,7 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
 
         strcatbuff(buffer, "&");
         while(s && (e = strchr(s, '=')) && (f = strchr(s, '&'))) {
-          char *ua;
+          const char *ua;
           String sua = STRING_EMPTY;
 
           *e = *f = '\0';
@@ -538,7 +506,7 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
           if (strfield2(ua, "on"))      /* hack : "on" == 1 */
             ua = "1";
           unescapehttp(ua, &sua);
-          inthash_write(NewLangList, s, (intptr_t) StringAcquire(&sua));
+          coucal_write(NewLangList, s, (intptr_t) StringAcquire(&sua));
           s = f + 1;
         }
       }
@@ -548,8 +516,8 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
         intptr_t adr = 0;
         intptr_t adr2 = 0;
 
-        if (inthash_readptr(NewLangList, "sid", &adr)) {
-          if (inthash_readptr(NewLangList, "_sid", &adr2)) {
+        if (coucal_readptr(NewLangList, "sid", &adr)) {
+          if (coucal_readptr(NewLangList, "_sid", &adr2)) {
             if (strcmp((char *) adr, (char *) adr2) != 0) {
               meth = 0;
             }
@@ -562,40 +530,40 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
         int doLoad = 0;
         intptr_t adr = 0;
 
-        if (inthash_readptr(NewLangList, "lang", &adr)) {
+        if (coucal_readptr(NewLangList, "lang", &adr)) {
           int n = 0;
 
           if (sscanf((char *) adr, "%d", &n) == 1 && n > 0
               && n - 1 != LANG_T(path, -1)) {
             LANG_T(path, n - 1);
             /* make a backup, because the GUI will override it */
-            inthash_write(NewLangList, "lang_",
+            coucal_write(NewLangList, "lang_",
                           (intptr_t) strdup((char *) adr));
           }
         }
 
         /* Load existing project settings */
-        if (inthash_readptr(NewLangList, "loadprojname", &adr)) {
+        if (coucal_readptr(NewLangList, "loadprojname", &adr)) {
           char *pname = (char *) adr;
 
           if (*pname) {
-            inthash_write(NewLangList, "projname", (intptr_t) strdup(pname));
+            coucal_write(NewLangList, "projname", (intptr_t) strdup(pname));
           }
-          inthash_write(NewLangList, "loadprojname", (intptr_t) NULL);
+          coucal_write(NewLangList, "loadprojname", (intptr_t) NULL);
           doLoad = 1;
-        } else if (inthash_readptr(NewLangList, "loadprojcateg", &adr)) {
+        } else if (coucal_readptr(NewLangList, "loadprojcateg", &adr)) {
           char *pname = (char *) adr;
 
           if (*pname) {
-            inthash_write(NewLangList, "projcateg", (intptr_t) strdup(pname));
+            coucal_write(NewLangList, "projcateg", (intptr_t) strdup(pname));
           }
-          inthash_write(NewLangList, "loadprojcateg", (intptr_t) NULL);
+          coucal_write(NewLangList, "loadprojcateg", (intptr_t) NULL);
         }
 
         /* intial configuration */
         {
-          if (!inthash_read(NewLangList, "conf_file_loaded", NULL)) {
-            inthash_write(NewLangList, "conf_file_loaded",
+          if (!coucal_read(NewLangList, "conf_file_loaded", NULL)) {
+            coucal_write(NewLangList, "conf_file_loaded",
                           (intptr_t) strdup("true"));
             doLoad = 2;
           }
@@ -605,8 +573,8 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
         if (!commandRunning) {
           intptr_t adrpath = 0, adrprojname = 0;
 
-          if (inthash_readptr(NewLangList, "path", &adrpath)
-              && inthash_readptr(NewLangList, "projname", &adrprojname)) {
+          if (coucal_readptr(NewLangList, "path", &adrpath)
+              && coucal_readptr(NewLangList, "projname", &adrprojname)) {
             StringClear(fspath);
             StringCat(fspath, (char *) adrpath);
             StringCat(fspath, "/");
@@ -646,7 +614,7 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
                 if (pos[0] == '0' && pos[1] == '\0')
                   *pos = '\0';  /* 0 => empty */
                 unescapeini(pos, &escline);
-                inthash_write(NewLangList, line,
+                coucal_write(NewLangList, line,
                               (intptr_t) StringAcquire(&escline));
               }
             }
@@ -662,7 +630,7 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
         intptr_t adr = 0;
         int p = 0;
 
-        if (inthash_readptr(NewLangList, "command", &adr)) {
+        if (coucal_readptr(NewLangList, "command", &adr)) {
           if (strcmp((char *) adr, "cancel") == 0) {
             if (commandRunning) {
               if (!commandEndRequested) {
@@ -706,15 +674,15 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
             if (!commandRunning) {
               intptr_t adrcd = 0;
 
-              if (inthash_readptr(NewLangList, "command_do", &adrcd)) {
+              if (coucal_readptr(NewLangList, "command_do", &adrcd)) {
                 intptr_t adrw = 0;
 
-                if (inthash_readptr(NewLangList, "winprofile", &adrw)) {
+                if (coucal_readptr(NewLangList, "winprofile", &adrw)) {
 
                   /* User general profile */
                   intptr_t adruserprofile = 0;
 
-                  if (inthash_readptr
+                  if (coucal_readptr
                       (NewLangList, "userprofile", &adruserprofile)
                       && adruserprofile != 0) {
                     int count = (int) strlen((char *) adruserprofile);
@@ -806,7 +774,7 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
           } else if (strcmp((char *) adr, "quit") == 0) {
             willexit = 1;
           }
-          inthash_write(NewLangList, "command", (intptr_t) NULL);
+          coucal_write(NewLangList, "command", (intptr_t) NULL);
         }
       }
 
@@ -818,7 +786,7 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
 
         if (url && *++url == '/' && (pos = strchr(url, ' ')) && !(*pos = '\0')) {
           char fsfile[1024];
-          char *file;
+          const char *file;
           FILE *fp;
           char *qpos;
 
@@ -859,7 +827,7 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
             } else {
               intptr_t adr = 0;
 
-              if (inthash_readptr(NewLangList, "projpath", &adr)) {
+              if (coucal_readptr(NewLangList, "projpath", &adr)) {
                 sprintf(fsfile, "%s%s", (char *) adr, file + 9);
               }
             }
@@ -880,7 +848,7 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
               "Server: httrack small server\r\n" "Content-type: text/plain\r\n";
 
             /* register current page */
-            inthash_write(NewLangList, "thisfile", (intptr_t) strdup(file));
+            coucal_write(NewLangList, "thisfile", (intptr_t) strdup(file));
 
             /* Force GET for the last request */
             if (meth == 2 && willexit) {
@@ -893,10 +861,10 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
                 "HTTP/1.0 302 Redirect\r\n" "Connection: close\r\n"
                 "Server: httrack-small-server\r\n";
               intptr_t adr = 0;
-              char *newfile = file;
+              const char *newfile = file;
 
-              if (inthash_readptr(NewLangList, "redirect", &adr) && adr != 0) {
-                char *newadr = (char *) adr;
+              if (coucal_readptr(NewLangList, "redirect", &adr) && adr != 0) {
+                const char *newadr = (char *) adr;
 
                 if (*newadr) {
                   newfile = newadr;
@@ -911,7 +879,7 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
                   StringMemcat(headers, tmp, strlen(tmp));
                 }
               }
-              inthash_write(NewLangList, "redirect", (intptr_t) NULL);
+              coucal_write(NewLangList, "redirect", (intptr_t) NULL);
             } else if (is_html(file)) {
               int outputmode = 0;
 
@@ -936,7 +904,7 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
                       && (n = (pos - str)) && n < 1024) {
                     char name_[1024 + 2];
                     char *name = name_;
-                    char *langstr = NULL;
+                    const char *langstr = NULL;
                     int p;
                     int format = 0;
                     int listDefault = 0;
@@ -964,13 +932,14 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
                       if (pos2 != NULL) {
                         *pos2 = '\0';
                         if (strstr(name, "..") == NULL) {
-                          if (fexist(fconcat(catbuff, path, name))) {
+                          if (fexist(fconcat(catbuff, sizeof(catbuff), path, name))) {
                             langstr = pos2 + 1;
                           }
                         }
                       }
                     } else if ((p = strfield(name, "do:"))) {
                       char *pos2;
+                      char empty[2] = "";
 
                       name += p;
                       format = 1;
@@ -980,7 +949,7 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
                         *pos2 = '\0';
                         pos2++;
                       } else {
-                        pos2 = "";
+                        pos2 = empty;
                       }
                       if (strcmp(name, "output-mode") == 0) {
                         if (strcmp(pos2, "html") == 0) {
@@ -994,34 +963,34 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
                         }
                       } else if (strcmp(name, "if-file-exists") == 0) {
                         if (strstr(pos2, "..") == NULL) {
-                          if (!fexist(fconcat(catbuff, path, pos2))) {
+                          if (!fexist(fconcat(catbuff, sizeof(catbuff), path, pos2))) {
                             outputmode = -1;
                           }
                         }
                       } else if (strcmp(name, "if-project-file-exists") == 0) {
                         if (strstr(pos2, "..") == NULL) {
                           if (!fexist
-                              (fconcat(catbuff, StringBuff(fspath), pos2))) {
+                              (fconcat(catbuff, sizeof(catbuff), StringBuff(fspath), pos2))) {
                             outputmode = -1;
                           }
                         }
                       } else if (strcmp(name, "if-file-do-not-exists") == 0) {
                         if (strstr(pos2, "..") == NULL) {
-                          if (fexist(fconcat(catbuff, path, pos2))) {
+                          if (fexist(fconcat(catbuff, sizeof(catbuff), path, pos2))) {
                             outputmode = -1;
                           }
                         }
                       } else if (strcmp(name, "if-not-empty") == 0) {
                         intptr_t adr = 0;
 
-                        if (!inthash_readptr(NewLangList, pos2, &adr)
+                        if (!coucal_readptr(NewLangList, pos2, &adr)
                             || *((char *) adr) == 0) {
                           outputmode = -1;
                         }
                       } else if (strcmp(name, "if-empty") == 0) {
                         intptr_t adr = 0;
 
-                        if (inthash_readptr(NewLangList, pos2, &adr)
+                        if (coucal_readptr(NewLangList, pos2, &adr)
                             && *((char *) adr) != 0) {
                           outputmode = -1;
                         }
@@ -1030,7 +999,7 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
                       } else if (strcmp(name, "loadhash") == 0) {
                         intptr_t adr = 0;
 
-                        if (inthash_readptr(NewLangList, "path", &adr)) {
+                        if (coucal_readptr(NewLangList, "path", &adr)) {
                           char *rpath = (char *) adr;
 
                           //find_handle h;
@@ -1043,9 +1012,9 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
                             const char *profiles = hts_getcategories(rpath, 0);
                             const char *categ = hts_getcategories(rpath, 1);
 
-                            inthash_write(NewLangList, "winprofile",
+                            coucal_write(NewLangList, "winprofile",
                                           (intptr_t) profiles);
-                            inthash_write(NewLangList, "wincateg",
+                            coucal_write(NewLangList, "wincateg",
                                           (intptr_t) categ);
                           }
                         }
@@ -1057,10 +1026,10 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
                             intptr_t adr = 0;
 
                             *pos3++ = '\0';
-                            if (inthash_readptr(NewLangList, pos2, &adr)) {
-                              inthash_write(NewLangList, pos3,
+                            if (coucal_readptr(NewLangList, pos2, &adr)) {
+                              coucal_write(NewLangList, pos3,
                                             (intptr_t) strdup((char *) adr));
-                              inthash_write(NewLangList, pos2, (intptr_t) NULL);
+                              coucal_write(NewLangList, pos2, (intptr_t) NULL);
                             }
                           }
                         }
@@ -1070,10 +1039,10 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
 
                           if (pos3) {
                             *pos3++ = '\0';
-                            inthash_write(NewLangList, pos2,
+                            coucal_write(NewLangList, pos2,
                                           (intptr_t) strdup(pos3));
                           } else {
-                            inthash_write(NewLangList, pos2, (intptr_t) NULL);
+                            coucal_write(NewLangList, pos2, (intptr_t) NULL);
                           }
                         }
                       }
@@ -1094,8 +1063,8 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
                       pos2 = strchr(name, ':');
                       if (pos2 != NULL) {
                         *pos2 = '\0';
-                        if (inthash_readptr(NewLangList, name, &adr) || ztest) {
-                          char *newadr = (char *) adr;
+                        if (coucal_readptr(NewLangList, name, &adr) || ztest) {
+                          const char *newadr = (char *) adr;
 
                           if (!newadr)
                             newadr = "";
@@ -1154,7 +1123,7 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
 
                           dname[0] = '\0';
                           strncatbuff(dname, name, n2);
-                          if (inthash_readptr(NewLangList, dname, &adr)) {
+                          if (coucal_readptr(NewLangList, dname, &adr)) {
                             int n = 0;
 
                             if (sscanf((char *) adr, "%d", &n) == 1) {
@@ -1171,15 +1140,15 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
                     if (langstr == NULL) {
                       if (strfield2(name, "#iso")) {
                         langstr = line2;
-                        langstr[0] = '\0';
-                        LANG_LIST(path, langstr);
+                        line2[0] = '\0';
+                        LANG_LIST(path, line2, sizeof(line2));
                         assertf(strlen(langstr) < sizeof(line2) - 2);
                       } else {
                         langstr = LANGSEL(name);
                         if (langstr == NULL || *langstr == '\0') {
                           intptr_t adr = 0;
 
-                          if (inthash_readptr(NewLangList, name, &adr)) {
+                          if (coucal_readptr(NewLangList, name, &adr)) {
                             char *newadr = (char *) adr;
 
                             langstr = newadr;
@@ -1191,7 +1160,7 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
                       switch (format) {
                       case 0:
                         {
-                          char *a = langstr;
+                          const char *a = langstr;
 
                           while(*a) {
                             if (a[0] == '\\' && isxdigit(a[1])
@@ -1237,7 +1206,7 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
                       default:
                         if (*langstr) {
                           int id = 1;
-                          char *fstr = langstr;
+                          const char *fstr = langstr;
 
                           StringClear(tmpbuff);
                           if (format == 2) {
@@ -1442,11 +1411,12 @@ int smallserver(T_SOC soc, char *url, char *method, char *data, char *path) {
 
 int htslang_init(void) {
   if (NewLangList == NULL) {
-    NewLangList = inthash_new(0);
+    NewLangList = coucal_new(0);
+    coucal_set_name(NewLangList, "NewLangList");
     if (NewLangList == NULL) {
       abortLog("Error in lang.h: not enough memory");
     } else {
-      inthash_value_is_malloc(NewLangList, 1);
+      coucal_value_is_malloc(NewLangList, 1);
     }
   }
   return 1;
@@ -1454,29 +1424,30 @@ int htslang_init(void) {
 
 int htslang_uninit(void) {
   if (NewLangList != NULL) {
-    inthash_delete(&NewLangList);
+    coucal_delete(&NewLangList);
   }
   return 1;
 }
 
-int smallserver_setkey(char *key, char *value) {
-  return inthash_write(NewLangList, key, (intptr_t) strdup(value));
+int smallserver_setkey(const char *key, const char *value) {
+  return coucal_write(NewLangList, key, (intptr_t) strdup(value));
 }
-int smallserver_setkeyint(char *key, LLint value) {
+
+int smallserver_setkeyint(const char *key, LLint value) {
   char tmp[256];
 
-  sprintf(tmp, LLintP, value);
-  return inthash_write(NewLangList, key, (intptr_t) strdup(tmp));
+  snprintf(tmp, sizeof(tmp), LLintP, value);
+  return coucal_write(NewLangList, key, (intptr_t) strdup(tmp));
 }
-int smallserver_setkeyarr(char *key, int id, char *key2, char *value) {
+int smallserver_setkeyarr(const char *key, int id, const char *key2, const char *value) {
   char tmp[256];
 
-  sprintf(tmp, "%s%d%s", key, id, key2);
-  return inthash_write(NewLangList, tmp, (intptr_t) strdup(value));
+  snprintf(tmp, sizeof(tmp), "%s%d%s", key, id, key2);
+  return coucal_write(NewLangList, tmp, (intptr_t) strdup(value));
 }
 
-static int htslang_load(char *limit_to, char *path) {
-  char *hashname;
+static int htslang_load(char *limit_to, const char *path) {
+  const char *hashname;
   char catbuff[CATBUFF_SIZE];
 
   //
@@ -1485,20 +1456,22 @@ static int htslang_load(char *limit_to, char *path) {
   //
   if (!limit_to) {
     LANG_DELETE();
-    NewLangStr = inthash_new(0);
-    NewLangStrKeys = inthash_new(0);
+    NewLangStr = coucal_new(0);
+    NewLangStrKeys = coucal_new(0);
+    coucal_set_name(NewLangStr, "NewLangStr");
+    coucal_set_name(NewLangStrKeys, "NewLangStrKeys");
     if ((NewLangStr == NULL) || (NewLangStrKeys == NULL)) {
       abortLog("Error in lang.h: not enough memory");
     } else {
-      inthash_value_is_malloc(NewLangStr, 1);
-      inthash_value_is_malloc(NewLangStrKeys, 1);
+      coucal_value_is_malloc(NewLangStr, 1);
+      coucal_value_is_malloc(NewLangStrKeys, 1);
     }
   }
 
   /* Load master file (list of keys and internal keys) */
   if (!limit_to) {
-    char *mname = "lang.def";
-    FILE *fp = fopen(fconcat(catbuff, path, mname), "rb");
+    const char *mname = "lang.def";
+    FILE *fp = fopen(fconcat(catbuff, sizeof(catbuff), path, mname), "rb");
 
     if (fp) {
       char intkey[8192];
@@ -1508,7 +1481,7 @@ static int htslang_load(char *limit_to, char *path) {
         linput_cpp(fp, intkey, 8000);
         linput_cpp(fp, key, 8000);
         if (strnotempty(intkey) && strnotempty(key)) {
-          char *test = LANGINTKEY(key);
+          const char *test = LANGINTKEY(key);
 
           /* Increment for multiple definitions */
           if (strnotempty(test)) {
@@ -1524,14 +1497,12 @@ static int htslang_load(char *limit_to, char *path) {
 
           if (!strnotempty(test)) {     // éviter doublons
             // conv_printf(key,key);
-            int len;
-            char *buff;
+            const size_t len = strlen(intkey);
+            char *const buff = (char *) malloc(len + 1);
 
-            len = (int) strlen(intkey);
-            buff = (char *) malloc(len + 2);
             if (buff) {
               strcpybuff(buff, intkey);
-              inthash_add(NewLangStrKeys, key, (intptr_t) buff);
+              coucal_add(NewLangStrKeys, key, (intptr_t) buff);
             }
           }
         }                       // if
@@ -1579,7 +1550,7 @@ static int htslang_load(char *limit_to, char *path) {
         hashname = LANGINTKEY(name);
       }
       sprintf(lbasename, "lang/%s.txt", hashname);
-      fp = fopen(fconcat(catbuff, path, lbasename), "rb");
+      fp = fopen(fconcat(catbuff, sizeof(catbuff), path, lbasename), "rb");
       if (fp) {
         char extkey[8192];
         char value[8192];
@@ -1588,9 +1559,7 @@ static int htslang_load(char *limit_to, char *path) {
           linput_cpp(fp, extkey, 8000);
           linput_cpp(fp, value, 8000);
           if (strnotempty(extkey) && strnotempty(value)) {
-            int len;
-            char *buff;
-            char *intkey;
+            const char *intkey;
 
             intkey = LANGINTKEY(extkey);
 
@@ -1598,7 +1567,7 @@ static int htslang_load(char *limit_to, char *path) {
 
               /* Increment for multiple definitions */
               {
-                char *test = LANGSEL(intkey);
+                const char *test = LANGSEL(intkey);
 
                 if (strnotempty(test)) {
                   if (loops == 0) {
@@ -1626,11 +1595,11 @@ static int htslang_load(char *limit_to, char *path) {
 
               /* Add key */
               if (strnotempty(intkey)) {
-                len = (int) strlen(value);
-                buff = (char *) malloc(len + 2);
+                const size_t len = strlen(value);
+                char *const buff = (char *) malloc(len + 1);
                 if (buff) {
                   conv_printf(value, buff);
-                  inthash_add(NewLangStr, intkey, (intptr_t) buff);
+                  coucal_add(NewLangStr, intkey, (intptr_t) buff);
                 }
               }
 
@@ -1652,7 +1621,7 @@ static int htslang_load(char *limit_to, char *path) {
 }
 
 /* NOTE : also contains the "webhttrack" hack */
-static void conv_printf(char *from, char *to) {
+static void conv_printf(const char *from, char *to) {
   int i = 0, j = 0, len;
 
   len = (int) strlen(from);
@@ -1721,12 +1690,12 @@ static void conv_printf(char *from, char *to) {
 }
 
 static void LANG_DELETE(void) {
-  inthash_delete(&NewLangStr);
-  inthash_delete(&NewLangStrKeys);
+  coucal_delete(&NewLangStr);
+  coucal_delete(&NewLangStrKeys);
 }
 
 // sélection de la langue
-static void LANG_INIT(char *path) {
+static void LANG_INIT(const char *path) {
   //CWinApp* pApp = AfxGetApp();
   //if (pApp) {
   /* pApp->GetProfileInt("Language","IntId",0); */
@@ -1734,7 +1703,7 @@ static void LANG_INIT(char *path) {
   //}
 }
 
-static int LANG_T(char *path, int l) {
+static int LANG_T(const char *path, int l) {
   if (l >= 0) {
     QLANG_T(l);
     htslang_load(NULL, path);
@@ -1742,7 +1711,7 @@ static int LANG_T(char *path, int l) {
   return QLANG_T(-1);           // 0=default (english)
 }
 
-static int LANG_SEARCH(char *path, char *iso) {
+static int LANG_SEARCH(const char *path, const char *iso) {
   char lang_str[1024];
   int i = 0;
   int curr_lng = LANG_T(path, -1);
@@ -1761,7 +1730,7 @@ static int LANG_SEARCH(char *path, char *iso) {
   return found;
 }
 
-static int LANG_LIST(char *path, char *buffer) {
+static int LANG_LIST(const char *path, char *buffer, size_t buffer_size) {
   char lang_str[1024];
   int i = 0;
   int curr_lng = LANG_T(path, -1);
@@ -1769,12 +1738,12 @@ static int LANG_LIST(char *path, char *buffer) {
   buffer[0] = '\0';
   do {
     QLANG_T(i);
-    strcpybuff(lang_str, "LANGUAGE_NAME");
+    strlcpybuff(lang_str, "LANGUAGE_NAME", buffer_size);
     htslang_load(lang_str, path);
     if (strlen(lang_str) > 0) {
       if (buffer[0])
         strcatbuff(buffer, "\n");
-      strcatbuff(buffer, lang_str);
+      strlcatbuff(buffer, lang_str, buffer_size);
     }
     i++;
   } while(strlen(lang_str) > 0);
@@ -1791,28 +1760,26 @@ static int QLANG_T(int l) {
   return lng;                   // 0=default (english)
 }
 
-static char *LANGSEL(char *name) {
-  intptr_t adr = 0;
-
-  if (NewLangStr)
-    if (!inthash_read(NewLangStr, name, &adr))
-      adr = 0;
-  if (adr) {
-    return (char *) adr;
+const char* LANGSEL(const char* name) {
+  coucal_value value;
+  if (NewLangStr != NULL 
+      && coucal_read_value(NewLangStr, name, &value) != 0
+      && value.ptr != NULL) {
+    return (char*) value.ptr;
+  } else {
+    return "";
   }
-  return "";
 }
 
-static char *LANGINTKEY(char *name) {
-  intptr_t adr = 0;
-
-  if (NewLangStrKeys)
-    if (!inthash_read(NewLangStrKeys, name, &adr))
-      adr = 0;
-  if (adr) {
-    return (char *) adr;
+const char* LANGINTKEY(const char* name) {
+  coucal_value value;
+  if (NewLangStrKeys != NULL
+      && coucal_read_value(NewLangStrKeys, name, &value) != 0
+      && value.ptr != NULL) {
+    return (char*) value.ptr;
+  } else {
+    return "";
   }
-  return "";
 }
 
 /* *** Various functions *** */
@@ -1845,7 +1812,7 @@ static int check_readinput(htsblk * r) {
     FD_SET(r->soc, &fds);
     tv.tv_sec = 0;
     tv.tv_usec = 0;
-    select(r->soc + 1, &fds, NULL, NULL, &tv);
+    select((int) r->soc + 1, &fds, NULL, NULL, &tv);
     if (FD_ISSET(r->soc, &fds))
       return 1;
     else
