@@ -1,7 +1,7 @@
 /* ------------------------------------------------------------ */
 /*
 HTTrack Website Copier, Offline Browser for Windows and Unix
-Copyright (C) 1998-2013 Xavier Roche and other contributors
+Copyright (C) 1998-2014 Xavier Roche and other contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -58,10 +58,9 @@ Please visit our Website: http://www.httrack.com
 #include "htsthread.h"
 
 /* External modules */
-#include "htsinthash.c"
+#include "coucal.c"
 #include "htsmd5.c"
 #include "md5.c"
-#undef HTS_PRINTF_FUN
 
 #include "htsserver.h"
 #include "htsweb.h"
@@ -179,6 +178,14 @@ int main(int argc, char *argv[]) {
   }
   smallserver_setkey("HTTRACK_WEB", HTTRACK_WEB);
 
+  /* Check version compatibility */
+  if (hts_sizeof_opt() != sizeof(httrackp)) {
+    fprintf(stderr,
+      "** CRITICAL: incompatible current httrack library version %s, expected version %s",
+      hts_version(), HTTRACK_VERSIONID);
+    smallserver_setkey("HTTRACK_INCOMPATIBLE_VERSIONID", hts_version());
+  }
+
   /* protected session-id */
   {
     char buff[1024];
@@ -239,7 +246,7 @@ static void back_launch_cmd(void *pP) {
   commandReturnCmdl = strdup(cmd);
 
   /* split */
-  argv[0] = "webhttrack";
+  argv[0] = strdup("webhttrack");
   argv[1] = cmd;
   argc++;
   i = 0;
@@ -265,6 +272,7 @@ static void back_launch_cmd(void *pP) {
   /* init */
   hts_init();
   global_opt = opt = hts_create_opt();
+  assert(opt->size_httrackp == sizeof(httrackp));
 
   /* run */
   commandReturn = webhttrack_runmain(opt, argc, argv);
