@@ -1,7 +1,7 @@
 /* ------------------------------------------------------------ */
 /*
 HTTrack Website Copier, Offline Browser for Windows and Unix
-Copyright (C) 1998-2013 Xavier Roche and other contributors
+Copyright (C) 1998-2014 Xavier Roche and other contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -69,6 +69,9 @@ static int linput(FILE * fp, char *s, int max);
 #include <unistd.h>
 #endif
 #include <ctype.h>
+#ifdef __linux
+#include <execinfo.h>
+#endif
 /* END specific definitions */
 
 static void __cdecl htsshow_init(t_hts_callbackarg * carg);
@@ -223,7 +226,17 @@ int main(int argc, char **argv) {
 
   signal_handlers();
   hts_init();
+
+  // Check version compatibility
+  if (hts_sizeof_opt() != sizeof(httrackp)) {
+    fprintf(stderr,
+      "incompatible current httrack library version %s, expected version %s",
+      hts_version(), HTTRACK_VERSIONID);
+    abortLog("incompatible httrack library version, please update both httrack and its library");
+  }
+
   opt = global_opt = hts_create_opt();
+  assert(opt->size_httrackp == sizeof(httrackp));
 
   CHAIN_FUNCTION(opt, init, htsshow_init, NULL);
   CHAIN_FUNCTION(opt, uninit, htsshow_uninit, NULL);
@@ -823,6 +836,7 @@ static void sig_ask(int code) { // demander
   }
 }
 #endif
+
 static void sig_brpipe(int code) {      // treat if necessary
   signal(code, sig_brpipe);
 }
@@ -860,6 +874,63 @@ static void sig_doback(int blind) {     // mettre en backing
 }
 #endif
 
+#undef FD_ERR
+#define FD_ERR 2
+
+static void print_backtrace(void) {
+#ifdef __linux
+  void *stack[256];
+  const int size = backtrace(stack, sizeof(stack)/sizeof(stack[0]));
+  if (size != 0) {
+    backtrace_symbols_fd(stack, size, FD_ERR);
+  }
+#else
+  const char msg[] = "No stack trace available on this OS :(\n";
+  write(FD_ERR, msg, sizeof(msg) - 1);
+#endif
+}
+
+static size_t print_num(char *buffer, int num) {
+  size_t i, j;
+  if (num < 0) {
+    *(buffer++) = '-';
+    num = -num;
+  }
+  for(i = 0 ; num != 0 || i == 0 ; i++, num /= 10) {
+    buffer[i] = '0' + ( num % 10 );
+  }
+  for(j = 0 ; j < i ; j++) {
+    const char c = buffer[i - j - 1];
+    buffer[i - j - 1] = buffer[j];
+    buffer[j] = c;
+  }
+  buffer[i] = '\0';
+  return i;
+}
+
+static void sig_fatal(int code) {
+  const char msg[] = "\nCaught signal ";
+  const char msgreport[] =
+    "\nPlease report the problem at http://forum.httrack.com\n";
+  char buffer[256];
+  size_t size;
+
+  signal(code, SIG_DFL);
+  signal(SIGABRT, SIG_DFL);
+
+  memcpy(buffer, msg, sizeof(msg) - 1);
+  size = sizeof(msg) - 1;
+  size += print_num(&buffer[size], code);
+  buffer[size++] = '\n';
+  (void) (write(FD_ERR, buffer, size) == size);
+  print_backtrace();
+  (void) (write(FD_ERR, msgreport, sizeof(msgreport) - 1)
+    == sizeof(msgreport) - 1);
+  abort();
+}
+
+#undef FD_ERR
+
 static void sig_leave(int code) {
   if (global_opt != NULL && global_opt->state._hts_in_mirror) {
     signal(code, sig_term);     // quitter si encore
@@ -896,6 +967,21 @@ static void signal_handlers(void) {
   signal(SIGPIPE, sig_brpipe);  // broken pipe (write into non-opened socket)
   signal(SIGCHLD, sig_ignore);  // child change status
 #endif
+#ifdef SIGABRT
+  signal(SIGABRT, sig_fatal);    // abort
+#endif
+#ifdef SIGBUS
+  signal(SIGBUS, sig_fatal);    // bus error
+#endif
+#ifdef SIGILL
+  signal(SIGILL, sig_fatal);    // illegal instruction
+#endif
+#ifdef SIGSEGV
+  signal(SIGSEGV, sig_fatal);   // segmentation violation
+#endif
+#ifdef SIGSTKFLT
+  signal(SIGSTKFLT, sig_fatal); // stack fault
+#endif
 }
 
 // fin routines de détournement de SIGHUP & co
